"""
flask-feide-gatekeeper demo flask app.

A minimal app showing off how the API Gatekeeper can be authenticated, and how
authentication data can be fetched from the request.

To run this API in flask:

::

    FLASK_APP=flask_feide_gk.demo_api flask run

"""
import logging

import flask

from . import basic_auth
from . import client
from . import gatekeeper
from . import utils
from . import proxyfix


logger = logging.getLogger(__name__)
app = flask.Flask(__name__)

# Dataporten api password
# app.config['FEIDE_BASIC_USERS'] = [('dataporten', '<password>')]

# Add localhost and dataporten gateway subnets to whitelist
# TODO: How should this be configured? It probably needs to be available before
#       the app context is...
loopback = ('127.0.0.0/8', '::1')
gatekeepers = ('158.36.86.32/27', '2001:700:0:4030::/61')
proxyfix.ProxyFix(loopback + gatekeepers, app=app)

# validator for credentials from the app config
creds = basic_auth.ConfigBackend(app)

# basic auth middleware
basic = basic_auth.BasicAuth(creds, app)

# feide gatekeeper headers
user = gatekeeper.GatekeeperData(basic)

# feide api client
feide_api = client.DataportenApi(user)


@app.before_request
def log_basic_request():
    """ log request rule match. """
    rule = flask.request.url_rule
    logger.debug("Got request for %r", rule)


@app.before_request
def log_remote_addr():
    logger.info("client-ip: %r", flask.request.remote_addr)
    logger.info("x-forwarded-for: %r",
                utils.get_multi_header('x-forwarded-for'))


@app.route('/')
def get_endpoints():
    """ list api endpoints. """
    def route_repr(rule):
        return {
            'path': str(rule.rule),
            'endpoint': str(rule.endpoint),
            'methods': tuple(map(str, rule.methods or ())),
            'arguments': tuple(map(str, rule.arguments or ())),
            'defaults': tuple(map(str, rule.defaults or ())),
        }

    return flask.jsonify(
        [route_repr(r)
         for r in flask.current_app.url_map.iter_rules()])


def _get_auth():
    return {
        'is_authenticated': basic.is_authenticated,
        'username': basic.username,
    }


@app.route('/is-authenticated')
def is_authenticated():
    """ Get authenticated username, if any. """
    return flask.jsonify(_get_auth())


@app.route('/require-basic')
@basic.require
def require_basic():
    """ Same as is_authenticated, but require basic auth. """
    return flask.jsonify(_get_auth())


@app.route('/gatekeeper-data')
@user.require
def get_dataporten_headers():
    """
    Get feide gatekeeper headers.

    This path requires gatekeeper basic auth, and valid gatekeeper headers. It
    returns the parsed gatekeeper header data.
    """
    return flask.jsonify({
        'client_id': user.client_id,
        'user_id': user.user_id,
        'user_sec': dict(user.user_sec) or None,
        'token': user.access_token,
    })


def format_response(r):
    return utils.format_block(
        utils.response_to_str(r, omit_body=True),
        prefix=" >  ",
        header="requests.Response",
        indent=2)


@app.route('/feide-api-data')
@user.require
def get_userinfo():
    """
    Get user info from the Feide API.

    It fetches user info from the Feide APIs using an access token provided by
    the Feide Gatekeeper.

    Note that this requires the appropriate scopes to be configured in the
    Gatekeeper for this API, *and* requires that the user asks for the
    appropriate scopes in the client.
    """
    with utils.upstream_request_handler("dataporten"):
        data = feide_api.get_user_info()

    logger.debug('Response:\n%s', data)
    return flask.jsonify(data)


@app.route('/feide-api-extended-data')
@user.require
def get_extended_userinfo():
    """
    Get extended user info from the Feide API.

    It fetches user info from the Feide APIs using an access token provided by
    the Feide Gatekeeper.

    Note that this requires the appropriate scopes to be configured in the
    Gatekeeper for this API, *and* requires that the user asks for the
    appropriate scopes in the client.
    """
    with utils.upstream_request_handler("dataporten"):
        data = feide_api.get_extended_user_info()

    logger.debug('Response:\n%s', data)
    return flask.jsonify(data)


def mime_to_name(name, mimetype):
    mimetype, _, params = mimetype.partition(';')
    mimetype, _, suffix = mimetype.partition('+')
    mimetype, _, subtype = mimetype.partition('/')
    if suffix:
        return name + '.' + suffix
    elif subtype:
        return name + '.' + subtype
    return name


@app.route('/feide-api-photo')
@user.require
def get_profile_photo():
    """
    Get user profile photo from the Feide API.
    """
    import io
    from flask import send_file
    with utils.upstream_request_handler("dataporten"):
        response = feide_api.get_user_photo()

    logger.debug('Response:\n%s', format_response(response))
    content_type = response.headers['Content-Type']
    content_file = io.BytesIO(response.content)
    content_name = mime_to_name('profile', content_type)

    return send_file(
        content_file,
        mimetype=content_type,
        as_attachment=True,
        attachment_filename=content_name)


logging.basicConfig(level=logging.DEBUG)
