"""
Support for fetching Feide (Dataporten) API Gatekeeper headers.
"""
import collections
import collections.abc
import functools
import logging

from flask import request as current_request
from werkzeug.exceptions import Forbidden

HEADER_CLIENT_ID = 'X-Dataporten-Clientid'
HEADER_USER_ID = 'X-Dataporten-Userid'
HEADER_USER_SEC = 'X-Dataporten-Userid-sec'
HEADER_TOKEN = 'X-Dataporten-token'

logger = logging.getLogger(__name__)


class UserIds(collections.abc.Mapping):
    """ Immutable mapping for dataporten user ids. """

    def __init__(self, *args, **kwargs):
        self.__user_ids = dict()
        for id_type, values in dict(*args, **kwargs).items():
            self.__user_ids[id_type] = tuple(values)

    def __len__(self):
        return len(self.__user_ids)

    def __iter__(self):
        return iter(self.__user_ids)

    def __getitem__(self, item):
        return self.__user_ids[item]

    def __repr__(self):
        return '{cls.__name__}({init})'.format(
            cls=type(self),
            init=repr(self.__user_ids))


def get_client_id(request):
    """ Extract dataporten client_id from the request. """
    # TODO: Validate is UUID?
    return request.headers.get(HEADER_CLIENT_ID)


def get_user_id(request):
    """ Extract dataporten user_id from the request. """
    # TODO: Validate is UUID?
    return request.headers.get(HEADER_USER_ID)


def get_user_sec(request):
    """ Extract secondary user IDs from the request.

    Example:

    >>> r, d = flask.Request({}), dict()
    >>> r.headers = d
    >>> get_user_sec(r)
    {}
    >>> d[USER_SEC_HEADER] = "foo:bar"
    >>> r.headers
    UserIds({'foo': ('bar',)})
    >>> d[USER_SEC_HEADER] = "foo:bar,foo:baz"
    >>> r.headers
    UserIds({'foo': ('bar', 'baz')})
    >>> d[USER_SEC_HEADER] = "foo:bar,baz:bar"
    >>> r.headers
    UserIds({'foo': ('bar',), 'baz': ('bar',)})
    """
    user_sec = collections.defaultdict(set)
    for sec in request.headers.get(HEADER_USER_SEC, '').split(','):
        source, sep, ident = sec.partition(':')
        if sep:
            user_sec[str(source)].add(str(ident))
    return UserIds(user_sec)


def get_access_token(request):
    """ Extract dataporten access token from the request. """
    # TODO: Validate is UUID?
    return request.headers.get(HEADER_TOKEN)


class GatekeeperData(object):
    """
    Flask middleware component for fetching Gatekeeper request data.

    Requests from a Gatekeeper *must* be validated using Basic auth.
    """
    def __init__(self, basic_auth, app=None):
        self.basic_auth = basic_auth
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        pass

    @property
    def is_authenticated(self):
        if not self.basic_auth.is_authenticated:
            return False
        missing = tuple((
            attr for attr in ('user_id', 'client_id')
            if not getattr(self, attr, None)))
        if missing:
            logger.debug('%s.is_authenticated: missing %r',
                         type(self).__name__, missing)
        return not missing

    @property
    def user_id(self):
        return get_user_id(current_request)

    @property
    def client_id(self):
        return get_client_id(current_request)

    @property
    def user_sec(self):
        return get_user_sec(current_request)

    @property
    def access_token(self):
        return get_access_token(current_request)

    def check_is_authenticated(self):
        self.basic_auth.check_is_authenticated()
        if not self.is_authenticated():
            logger.warning('%s.require: missing auth', type(self).__name__)
            raise Forbidden()

    def require(self, func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            self.check_is_authenticated()
            return func(*args, **kwargs)
        return wrapper
